@@ -5,9 +5,6 @@
 #include <config.h>
 #endif
 
-#include <sys/types.h>
-#include <sane/sane.h>
-
 #include "epkowa.h"
 #include "sane/sanei_scsi.h"
 
