@@ -1,5 +1,5 @@
 /* file-selector.cc -- customized file selection dialog
-   Copyright (C) 2003, 2005  SEIKO EPSON Corporation
+   Copyright (C) 2003, 2005, 2008  SEIKO EPSON CORPORATION
 
    This file is part of the `iscan' program.
 
@@ -35,16 +35,21 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
 
+
 #include "pisa_aleart_dialog.h"
 #include "pisa_enums.h"
 #include "pisa_error.h"
 
-#include "../lib/imgstream.hh"
+#include "../lib/pcxstream.hh"
 #include "../lib/pnmstream.hh"
 #include "../lib/pngstream.hh"
 #include "../lib/jpegstream.hh"
+#include "../lib/tiffstream.hh"
+#include "../lib/pdfstream.hh"
 
 #ifndef DEBUG
 #define g_print(...)
@@ -75,6 +80,7 @@ gtk_widget_get_parent (GtkWidget *widget)
 struct menu_info		// helper struct for callbacks
 {
   file_selector *fs;
+  GtkWidget     *widget[4];     // holding "ADF" spinboxes and labels
   int            index;
 };
 
@@ -111,46 +117,34 @@ _click_ok (file_selector *fs)
     }
 }
 
-#if 0
-#ifndef HAVE_GTK_2
-static void
-_select_filename( GtkCList *clist, gint row, gint column, GdkEventButton *,
-		  file_selector *fs )
-{
-  DBG_FS fprintf( stderr, "_select_filename\n" );
-
-  char *text;
-  gtk_clist_get_text( clist, row, column, &text );
-
-  fs->set_filename( text );
-}
-#endif
-
-static void
-_change_filename( GtkEditable *editable, file_selector *fs )
-{
-  DBG_FS fprintf( stderr, "_change_filename\n" );
-
-  gchar *name = gtk_editable_get_chars( editable, 0, -1 );
-
-  gint cursor = gtk_editable_get_position( editable );
-  gtk_signal_handler_block_by_func( GTK_OBJECT( editable ),
-				    GTK_SIGNAL_FUNC( _change_filename ),
-				    fs );
-  fs->set_filename( name, true );
-  gtk_signal_handler_unblock_by_func( GTK_OBJECT( editable ),
-				      GTK_SIGNAL_FUNC( _change_filename ),
-				      fs );
-  gtk_editable_set_position( editable, cursor );
-  
-  g_free( name );
-}
-#endif
-
+// FIXME: this really should be using image stream capability API to
+//        determine what is and what is not supported.
 static void
 _change_format (GtkWidget *, struct menu_info *id)
 {
-  id->fs->change_format( id->index );
+  id->fs->change_format( id->index ); // fs->get_type() below, requires this
+
+  // adjust "file with all pages" check box
+  bool make_active = (iscan::PDF == id->fs->get_type () ||
+                      iscan::TIF == id->fs->get_type ());
+  gtk_widget_set_sensitive (id->widget[0], make_active);
+  if (!make_active) // disable multi-page mode when changing to a format that
+                    // does not support it, do not do the opposite!
+    {
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (id->widget[0]), false);
+    }
+
+  // adjust "binding edge" selector sensitivity
+  // We only support this for file formats (i.e. `imgstream` classes)
+  // that support "easy" rotation.  Yes, this is utterly flawed.  The
+  // "binding edge" selector _should_ be in the main part of the GUI.
+  // It has no dependency on saving to file whatsoever, it is an issue
+  // that depends on device capabilities and document characteristics
+  // only.
+  make_active = (iscan::PDF == id->fs->get_type ());
+  gtk_widget_set_sensitive (id->widget[1], make_active);
+  gtk_widget_set_sensitive (id->widget[2], make_active);
+  gtk_widget_set_sensitive (id->widget[3], make_active);
 }
 
 static void
@@ -206,17 +200,42 @@ _change_digits (GtkAdjustment *, file_selector *fs)
   fs->change_digits ();
 }
 
+static void
+_toggled_multi_page_mode_fs (GtkWidget *widget, file_selector *fs)
+{
+  fs->change_page_mode (gtk_toggle_button_get_active (
+                        GTK_TOGGLE_BUTTON (widget)));
+}
+
+static void
+_changed_binding_edge (GtkWidget *widget, file_selector *fs)
+{
+  fs->change_binding_edge (gtk_toggle_button_get_active (
+                           GTK_TOGGLE_BUTTON (widget)));
+}
+static void
+_toggled_multi_page_mode_widget (GtkWidget *widget, GtkWidget *p)
+{
+  bool on = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+  gtk_widget_set_sensitive (p, !on);
+}
 
 // 	class implementation
 
+using std::string;
+using std::stringstream;
+using iscan::file_opener;
+
 			 	// define static variables
 const file_selector::format
 file_selector::_format[]  = {
-  { "PNM", 0, "pnm", "pnm" },
-  { "PNG", 0, "png", "png" },
-  { "JPEG", 0, "jpg", "jpg|jpeg" },
-  { "TIFF", 0, "tiff", "tiff" },
-  { 0 }				// array terminator
+  { "PCX", 0, "pcx", "pcx", iscan::PCX },
+  { "PNM", 0, "pnm", "pnm", iscan::PNM },
+  { "PNG", 0, "png", "png", iscan::PNG },
+  { "JPEG", 0, "jpg", "jpg|jpeg", iscan::JPG },
+  { "TIFF", 0, "tiff", "tif|tiff", iscan::TIF },
+  { "PDF", 0, "pdf", "pdf", iscan::PDF },
+  { 0, 0, 0, 0, iscan::PNM }    // array terminator
 };
 
 void
@@ -229,8 +248,10 @@ file_selector::init ()
   _pathname = NULL;
 
   _using_adf = false;
-  _file_type = 0;
+  _format_index = 0;
   _deleted_all = false;
+  _multi_page_mode = false;
+  _has_left_edge_binding = true;
 
   _number = -1;
   _seqnum = NULL;
@@ -242,6 +263,8 @@ file_selector::init ()
   _fmt = new const format * [size];
 				// okay, so we may waste a few
 
+  // Looks like the idea is to only add supported formats to _fmt and
+  // leave unsupported formats out.
   int cnt = 0;
   int len = 0;
   for (int i = 0; i < size - 1; ++i)
@@ -266,24 +289,17 @@ file_selector::init ()
   g_print ("%s: ext_regex =`%s'\n", __func__, _ext_regex);
 }
 
-void
-file_selector::create_window( GtkWidget *parent, int option,
-			      bool enable_start_button, bool monochrome)
-{
-  create_window (GTK_WINDOW (gtk_widget_get_parent (parent)), parent,
-		 (   PISA_OP_ADF     == option
-		  || PISA_OP_ADFDPLX == option
-		  || enable_start_button));
-}
-
 void
 file_selector::create_window (GtkWindow *window, GtkWidget *parent,
-			      bool do_consecutive_scanning)
+			      bool do_consecutive_scanning,
+			      bool do_duplex,
+			      const char* default_format)
 {
   g_print ("%s: enter\n", __func__);
 
   _parent    = parent;
   _using_adf = do_consecutive_scanning;
+  _do_duplex = do_duplex;
 
   _widget = GTK_FILE_SELECTION (gtk_file_selection_new (PACKAGE));
 
@@ -292,7 +308,14 @@ file_selector::create_window (GtkWindow *window, GtkWidget *parent,
 
   gtk_window_set_transient_for (GTK_WINDOW (_widget), window);
   gtk_file_selection_hide_fileop_buttons (_widget);
-  add_dialog_extensions ();
+
+  gint index = 0;
+  if (is_supported (default_format))
+    index = get_index (default_format);
+  else if (is_supported ("PNG"))
+    index = get_index ("PNG");
+
+  add_dialog_extensions (index);
 				// needs to be called before we canonize
 
   if (!_filename)
@@ -370,7 +393,7 @@ file_selector::destroy (bool really)
   _digits =  NULL;
 
   _using_adf = false;
-  _file_type = 0;
+  _do_duplex = false;
   _deleted_all = false;
 
   delete [] _fmt;
@@ -380,74 +403,40 @@ file_selector::destroy (bool really)
 }
 
 char *
-file_selector::get_filename () const
+file_selector::get_pathname () const
 {
   return (_pathname ? strdup (_pathname) : NULL);
 }
 
-bool
-file_selector::set_filename( const char *text, bool edit )
-{
-  g_print ("file_selector::set_filename( %s, %d )\n", text, edit);
-
-  if (!_filename)
-    return false;		// logical error but return for now
-
-  if (!text
-      || !*text			// empty string
-      || text == _filename || 0 == strcmp( text, _filename ))
-    return false;		// nothing to change
-
-  g_print ("filename bfore = `%s'\n", _filename);
-  g_print ("filename inGUI = `%s'\n",
-	   gtk_file_selection_get_filename( _widget ));
-
-  char *old_name  = _filename;	// hang on to original
-#ifdef HAVE_GTK_2
-  {				// editing file extension not allowed
-    int n = strrchr( _filename, '.') - _filename;
-    edit = (0 == strncmp (text, _filename, n));
-  }
-#endif
-  _filename = (edit ? validate( text ) : canonize( text ));
-
-  if (_filename)		// we got a new name
-    free( old_name );
-  else
-    _filename = old_name;	// revert to original
-
-				// update the GUI
-  gtk_file_selection_set_filename( _widget, _filename );
-
-  g_print ("filename after = `%s'\n", _filename );
-  g_print ("filename inGUI = `%s'\n",
-	   gtk_file_selection_get_filename( _widget ) );
-
-  g_print ("set_filename returns %d\n", _filename != old_name );
-  return _filename != old_name;
-}
-
 int
 file_selector::get_sequence_number () const
 {
   return _number;
 }
 
-void
-file_selector::set_sequence_number (int number)
+bool
+file_selector::valid_pathname (const string& path) const
 {
-  if (!_using_adf || !_seqnum)
-    return;
+  string::size_type slash = path.rfind (file_opener::dir_sep);
+  string name = path.substr (slash + 1);
+  string::size_type dot = name.rfind (file_opener::ext_sep);
 
-  _number = number;
+  string::size_type cmp_pos = dot;
 
-  while (_number > _seqnum->upper)
+  if (_using_adf && !_multi_page_mode)
     {
-      int max = int (_seqnum->upper) * 10 + 9;
-      _seqnum->upper = max;
+      string::size_type dash = name.find_last_of ("-", dot - 1);
+      cmp_pos = dash;
     }
-  gtk_adjustment_set_value (_seqnum, _number);
-  gtk_adjustment_changed (_seqnum);
+
+  if (128 < path.size ()	// blame the spec for this one
+      || !permission (path.c_str ())
+      || 0 >= cmp_pos) // 0 length filename
+    {
+      return false;
+    }
+
+  return true;
 }
 
 bool
@@ -465,15 +454,15 @@ file_selector::save_pathname()
 
   const char *current = gtk_file_selection_get_filename (_widget);
 
-  if (128 < strlen (current)	// blame the spec for this one
-      || !permission (current))
+  if (!valid_pathname (current))
     {
       show_message (pisa_error (PISA_ERR_FILENAME));
       return false;
     }
-  if (0 == access (current, F_OK))
+  if (possible_overwrite (current))
     {
-      bool ok = show_message (pisa_error (PISA_ERR_OVERWRITE), true);
+      bool ok = show_message (pisa_error (PISA_ERR_OVERWRITE),
+                              _("Overwrite"), _("Cancel"));
       if (!ok)
 	return false;
     }
@@ -565,7 +554,8 @@ file_selector::delete_text (gint start, gint end)
   _deleted_all = ((end - start) == (gint) strlen (_filename));
 
   if (0 > end) end = strlen (_filename) + 1;
-  int dot = strrchr (_filename, (_using_adf ? '-' : '.')) - _filename;
+  int dot = strrchr (_filename, ((_using_adf && !_multi_page_mode)
+                                 ? '-' : '.')) - _filename;
   if (end > dot) end = dot;
 
   if (start < end)
@@ -588,6 +578,7 @@ file_selector::insert_text (gchar *text, gint len, gint *pos)
     {
       _deleted_all = false;
       text = canonize (text);
+      if (!text) return;
       len  = strlen (text);
 
       GtkEditable *ed = GTK_EDITABLE (_widget->selection_entry);
@@ -604,7 +595,8 @@ file_selector::insert_text (gchar *text, gint len, gint *pos)
 #endif
     }
 
-  int dot = strrchr (_filename, (_using_adf ? '-' : '.')) - _filename;
+  int dot = strrchr (_filename, ((_using_adf && !_multi_page_mode)
+                                 ? '-' : '.')) - _filename;
 
   if (pos && (*pos <= dot))
     {
@@ -630,7 +622,7 @@ file_selector::change_format (int index)
 {
   g_print ("%s (%d)\n", __func__, index);
 
-  _file_type = index;
+  _format_index = index;
 				// reflect changes in filename
   char *canon = canonize (_filename);
   set_entry (canon);
@@ -674,61 +666,44 @@ file_selector::change_digits ()
     }
 }
 
-// caller needs to free returned char *
-char *
-file_selector::validate (const char *text) const
+#include <sstream>
+#include <iostream>
+void
+file_selector::change_page_mode (bool mode)
 {
-  if (!text || 0 == *text)
-    return 0;
+  _multi_page_mode = mode;
+  
+  string name = _filename;
+  stringstream ss;
+  
+  string::size_type dot  = name.rfind (file_opener::ext_sep);
+  string::size_type dash = name.find_last_of ("-", dot - 1);
 
-  g_print ("%s (%s)\n", __func__, text);
-
-  char *valid = 0;
-
-  // match extension with filetype
-  // match number of #'s with number of digits iff using ADF
-
-  const char *regex_fmt = "^(.+)(-#{%d}){%d}\\.(%s)$";
-  const char *ext_regex = _fmt[_file_type]->rgx;
-  char *regex = new char[strlen( regex_fmt ) - 3	// 3 formatters
-			 + strlen( ext_regex ) + 1];	// final '\0'
-  sprintf(regex, regex_fmt,
-	  (_digits    ? int (_digits->value) : 0),
-	  (_using_adf ?      1               : 0),
-	  ext_regex);
-
-  regex_t *comp_regex = new regex_t;
-  int comp = regcomp (comp_regex, regex, REG_EXTENDED);
-
-  if (0 == comp)
+  if (mode)
     {
-      int result = regexec (comp_regex, text, 0, 0, 0);
-      if (0 == result)
-	{
-	  valid = (char *) malloc ((strlen (text) + 1)
-				   * sizeof (char));
-	  if (valid)
-	    strcpy (valid, text);
-	  else
-	    {			// FIXME!
-	      // no memory
-	    }
-	}
-      else
-	if (REG_NOMATCH != result)
-	  regerror (comp, comp_regex);
+      ss << name.substr (0, dash)
+         << name.substr (dot);
     }
   else
-    regerror (comp, comp_regex);
-
-  regfree (comp_regex);
-  delete comp_regex;
-  delete[] regex;
-
-  return valid;
+    {
+      ss << name.substr (0, dot) << "-";
+      for (int i = 0; i < int (_digits->value); ++i)
+        ss << file_opener::hash_mark;
+      ss << name.substr (dot);
+    }
+      
+  set_entry (ss.str ().c_str ());
 }
 
-// caller needs to free returned char *, because we validate()
+void
+file_selector::change_binding_edge (bool is_left_edge)
+{
+  g_print ("%s(%d)\n", __func__, is_left_edge);
+
+  _has_left_edge_binding = is_left_edge;
+}
+
+// caller needs to free returned char *
 char *
 file_selector::canonize (const char *text) const
 {
@@ -737,177 +712,70 @@ file_selector::canonize (const char *text) const
 
   g_print ("%s (%s)\n", __func__, text);
 
-  char *interim = 0;
+  string name = text;
+  string ext = _fmt[_format_index]->ext;
+  stringstream ss;
 
-  {	    // "replace" existing extension with one matching filetype
-    const char * ext_regex_fmt = "^(.+)\\.(%s)$";
-    char *regex = new char[strlen (ext_regex_fmt) - 1
-			   + strlen (_ext_regex) + 1];
-    sprintf (regex, ext_regex_fmt, _ext_regex);
+  string::size_type dot = name.rfind (file_opener::ext_sep);
 
-    regex_t *comp_regex = new regex_t;
-    int comp = regcomp (comp_regex, regex, REG_EXTENDED);
+  if (_using_adf && !_multi_page_mode)
+    {
+      string::size_type dash = name.find_last_of ("-", dot - 1);
+      ss << name.substr (0, dash)
+         << "-";
+      
+      for (int i = 0; i < int (_digits->value); ++i)
+        ss << file_opener::hash_mark;
+    }
+  else
+    {
+      ss << name.substr (0, dot);
+    }
 
-    if (0 == comp)
-      {
-	char *file_ext = _fmt[_file_type]->ext;
+  ss << file_opener::ext_sep
+     << ext;
 
-	size_t      nsub  = comp_regex->re_nsub + 1;
-	regmatch_t *match = new regmatch_t[nsub];
-
-	int result = regexec (comp_regex, text, nsub, match, 0);
-	if (0 == result)
-	  {			// replace existing extension
-	    // FIXME: keep extension if in _fmt[_file_type]->rgx
-	    int pre = 1;
-
-	    regoff_t l_pre = match[pre].rm_eo - match[pre].rm_so;
-
-	    size_t len = l_pre + strlen (file_ext) + 2;
-				// period and terminating '\0'
-	    interim = (char *) malloc (len * sizeof (char));
-	    if (interim)
-	      {
-		char *c = interim;
-		{		// copy prefix
-		  const char *p = text + match[pre].rm_so;
-		  while (0 < l_pre--)
-		    *c++ = *p++;
-		}
-		{		// append extension
-		  *c++ = '.';
-		  while (*file_ext)
-		    *c++ = *file_ext++;
-		  *c++ = '\0';
-		}
-	      }
-	    else
-	      {			// FIXME!
-		// no memory
-	      }
-	  } // 0 == result
-	else
-	  if (REG_NOMATCH == result)
-	    {			// append a new extension
-	      size_t len = strlen (text) + strlen (file_ext) + 2;
-				// period and terminating '\0'
-	      interim = (char *) malloc (len * sizeof (char));
-	      if (interim)
-		sprintf (interim, "%s.%s", text, file_ext);
-	      else
-		{		// FIXME!
-		  // no memory
-		}
-	    }
-	  else
-	    regerror (result, comp_regex);
-	delete[] match;
-      }	// 0 == comp
-    else
-      regerror (comp, comp_regex);
-
-    regfree (comp_regex);
-    delete comp_regex;
-    delete[] regex;
-  }
-
-  // FIXME: free interim if one of the new's throws
-
-  if (interim && _using_adf)
-    {	   // adjust the number of #'s to the current number of digits
-
-      const char * adf_regex_fmt = "^(((.+)-#+)|(.+))\\.(%s)$";
-      char *regex = new char[strlen (adf_regex_fmt) - 1
-			     + strlen (_ext_regex) + 1];
-      sprintf (regex, adf_regex_fmt, _ext_regex);
-
-      regex_t *comp_regex = new regex_t;
-      int comp = regcomp (comp_regex, regex, REG_EXTENDED);
-
-      if (0 == comp)
-	{
-	  size_t      nsub  = comp_regex->re_nsub + 1;
-	  regmatch_t *match = new regmatch_t[nsub];
-
-	  int result = regexec (comp_regex, interim, nsub, match, 0);
-	  if (0 == result)
-	    {			// see adf_regex_fmt
-	      int pre = (-1 == match[4].rm_so
-			 ? 3	// already contains sequence template
-			 : 4);
-	      int ext = 5;
-
-	      regoff_t l_pre = match[pre].rm_eo - match[pre].rm_so;
-	      regoff_t l_ext = match[ext].rm_eo - match[ext].rm_so;
-
-	      size_t len = l_pre + l_ext + int( _digits->value ) + 3;
-				// hyphen, period and terminating '\0'
-	      char *name = (char *) malloc (len * sizeof (char));
-	      if (name)
-		{
-		  char *c = name;
-		  {		// copy prefix
-		    const char *p = interim + match[pre].rm_so;
-		    while (0 < l_pre--)
-		      *c++ = *p++;
-		  }
-		  {		// insert -### part
-		    *c++ = '-';
-		    for (int i = 0; i < int (_digits->value); ++i)
-		      *c++ = '#';
-		  }
-		  {		// append extension
-		    *c++ = '.';
-		    const char *p = interim + match[ext].rm_so;
-		    while (0 < l_ext--)
-		      *c++ = *p++;
-		  }
-		  *c = '\0';
-		} // name
-	      else
-		{		// FIXME!
-		  // no memory
-		}
-	      free (interim);
-	      interim = name;
-	    } // 0 == result
-	  else
-	    if (REG_NOMATCH != result)
-	      regerror (result, comp_regex);
-	  delete[] match;
-	} // 0 == comp
-      else
-	regerror (comp, comp_regex);
-
-      regfree (comp_regex);
-      delete comp_regex;
-      delete[] regex;
-    } // _using_adf
-
-  char *result = validate (interim);
-  free (interim);		// clean up what we allocated
-
-  return result;
+  return strdup (ss.str ().c_str ());
 }
 
 void
-file_selector::add_dialog_extensions ()
+file_selector::add_dialog_extensions (gint index)
 {
   g_print ("%s: enter\n", __func__);
 
   GtkWidget *frame = gtk_frame_new (_("Save Options"));
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
 
+  GtkWidget *align = gtk_alignment_new (1.0, 0.5, 0.0, 0.0);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (align), 5, 5, 5, 5);
+  gtk_container_add (GTK_CONTAINER (frame), GTK_WIDGET (align));
+  gtk_widget_show (align);
+
+  unsigned short rows = 1;
+  if (_using_adf) rows = 2;
+  if (_do_duplex) rows = 3;
   GtkTable *opts
-    = (GtkTable *) gtk_table_new ((_using_adf ? 3 : 1), 2, TRUE);
-  gtk_container_add (GTK_CONTAINER (frame), GTK_WIDGET (opts));
+    = (GtkTable *) gtk_table_new (rows + (_using_adf ? 2 : 0), 3, FALSE);
+  gtk_table_set_col_spacings (opts, 5);
+  gtk_container_add (GTK_CONTAINER (align), GTK_WIDGET (opts));
+  gtk_widget_show (GTK_WIDGET (opts));
+
+  GtkWidget *mi = NULL;
+  GtkWidget *multi =
+    gtk_check_button_new_with_label ( _("Create file with all pages"));
+
+  GtkWidget *bind_edge = gtk_label_new
+    (_("Binding Position (for Double-Sided Scanning)"));
+  GtkWidget *left_edge = gtk_radio_button_new_with_label (NULL, _("Left"));
+  GtkWidget *top_edge  = gtk_radio_button_new_with_label
+    (gtk_radio_button_get_group (GTK_RADIO_BUTTON (left_edge)), _("Top"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (left_edge), true);
 
   GtkWidget *w = 0;
   {				// file type selector
     w = gtk_label_new (_("Determine File Type:"));
-    gtk_label_set_justify (GTK_LABEL (w), GTK_JUSTIFY_LEFT);
     gtk_table_attach_defaults (opts, w, 0, 1, 0, 1);
-    gtk_misc_set_alignment (GTK_MISC (w), 0.1, 0.5);
+    gtk_misc_set_alignment (GTK_MISC (w), 1.0, 0.5);
     gtk_widget_show (w);
 
     w = gtk_option_menu_new ();
@@ -916,41 +784,28 @@ file_selector::add_dialog_extensions ()
 
       for (int i = 0; _fmt[i]; ++i)
 	{
-	  GtkWidget *mi = gtk_menu_item_new_with_label (_fmt[i]->name);
+	  mi = gtk_menu_item_new_with_label (_fmt[i]->name);
 	  gtk_menu_append (GTK_MENU (m), mi);
 	  gtk_widget_show (mi);
 
 	  struct menu_info *id = new struct menu_info;
 	  id->fs = this;
+	  id->widget[0] = multi;
+	  id->widget[1] = bind_edge;
+	  id->widget[2] = left_edge;
+	  id->widget[3] = top_edge;
 	  id->index = i;
 
 	  g_signal_connect (GTK1_OBJ (mi), "activate",
 			    G_CALLBACK (_change_format), (void *) id);
 
-	  // FIXME! ugly kludge to "check" for support
-	  if (0 == strcmp ("PNM", _fmt[i]->name))
-	    {
-	      gtk_widget_set_sensitive (mi, iscan::pnmstream::is_usable ());
-	    }
-	  if (0 == strcmp ("PNG", _fmt[i]->name))
-	    {
-	      gtk_widget_set_sensitive (mi, iscan::pngstream::is_usable ());
-	    }
-	  if (0 == strcmp ("JPEG", _fmt[i]->name))
-	    {
-	      gtk_widget_set_sensitive (mi, iscan::jpegstream::is_usable ());
-	    }
-	  // FIXME: this is only here to show what the GUI would look like
-	  //	TIFF support is planned but was not ready in time for this
-	  //	release :-(
-	  if (0 == strcmp ("TIFF", _fmt[i]->name))
-	    {
-	      gtk_widget_set_sensitive (mi, false);
-	    }
+          gtk_widget_set_sensitive (mi, is_supported (_fmt[i]->name));
 	}
       gtk_option_menu_set_menu( GTK_OPTION_MENU( w ), m );
     }
-    gtk_table_attach_defaults (opts, w, 1, 2, 0, 1);
+    gtk_table_attach_defaults (opts, w, 1, 3, 0, 1);
+    gtk_option_menu_set_history (GTK_OPTION_MENU (w), index);
+    change_format (index);
     gtk_widget_show (w);
   }
 
@@ -958,10 +813,30 @@ file_selector::add_dialog_extensions ()
     {
       g_print ("%s: using adf\n", __func__);
 
+      gtk_signal_connect (GTK_OBJECT (multi), "toggled",
+      	            GTK_SIGNAL_FUNC (_toggled_multi_page_mode_fs), this);
+      gtk_table_attach_defaults (opts, multi, 1, 3, 1, 2);
+      gtk_widget_show (multi);
+
+      if (_do_duplex)
+        {
+          gtk_signal_connect (GTK_OBJECT (left_edge), "toggled",
+                              GTK_SIGNAL_FUNC (_changed_binding_edge), this);
+          gtk_table_attach_defaults (opts, bind_edge, 0, 1, 2, 3);
+          gtk_misc_set_alignment (GTK_MISC (bind_edge), 1.0, 0.5);
+          gtk_table_attach_defaults (opts, left_edge, 1, 2, 2, 3);
+          gtk_table_attach_defaults (opts, top_edge, 2, 3, 2, 3);
+          gtk_widget_show (bind_edge);
+          gtk_widget_show (left_edge);
+          gtk_widget_show (top_edge);
+        }
+
  				// sequence number selector
       w = gtk_label_new (_("Start filing at:"));
-      gtk_table_attach_defaults (opts, w, 0, 1, 1, 2);
-      gtk_misc_set_alignment (GTK_MISC (w), 0.1, 0.5);
+      gtk_table_attach_defaults (opts, w, 0, 1, rows + 0, rows + 1);
+      gtk_misc_set_alignment (GTK_MISC (w), 1.0, 0.5);
+      gtk_signal_connect (GTK_OBJECT (multi), "toggled",
+              GTK_SIGNAL_FUNC (_toggled_multi_page_mode_widget), w);
       gtk_widget_show (w);
 
       if (!_seqnum)
@@ -974,13 +849,17 @@ file_selector::add_dialog_extensions ()
       w = gtk_spin_button_new (_seqnum, 0, 0);
       gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (w), true);
       gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (w), true);
-      gtk_table_attach_defaults (opts, w, 1, 2, 1, 2);
+      gtk_table_attach_defaults (opts, w, 1, 3, rows + 0, rows + 1);
+      gtk_signal_connect (GTK_OBJECT (multi), "toggled",
+              GTK_SIGNAL_FUNC (_toggled_multi_page_mode_widget), w);
       gtk_widget_show (w);
 
 				// number of digits selector
       w = gtk_label_new (_("Number of digits:"));
-      gtk_table_attach_defaults (opts, w, 0, 1, 2, 3);
-      gtk_misc_set_alignment (GTK_MISC (w), 0.1, 0.5);
+      gtk_table_attach_defaults (opts, w, 0, 1, rows + 1, rows + 2);
+      gtk_misc_set_alignment (GTK_MISC (w), 1.0, 0.5);
+      gtk_signal_connect (GTK_OBJECT (multi), "toggled",
+              GTK_SIGNAL_FUNC (_toggled_multi_page_mode_widget), w);
       gtk_widget_show (w);
 
       if (!_digits)
@@ -991,7 +870,9 @@ file_selector::add_dialog_extensions ()
       w = gtk_spin_button_new (_digits, 0, 0);
       gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (w), true);
       gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (w), true);
-      gtk_table_attach_defaults (opts, w, 1, 2, 2, 3);
+      gtk_table_attach_defaults (opts, w, 1, 3, rows + 1, rows + 2);
+      gtk_signal_connect (GTK_OBJECT (multi), "toggled",
+              GTK_SIGNAL_FUNC (_toggled_multi_page_mode_widget), w);
       gtk_widget_show (w);
 
       g_signal_connect (GTK1_OBJ (_seqnum), "value_changed",
@@ -1002,23 +883,37 @@ file_selector::add_dialog_extensions ()
 
   gtk_box_pack_end (GTK_BOX (_widget->main_vbox), frame, false, false, 0);
 
+  struct menu_info id;
+  id.fs = this;
+  id.widget[0] = multi;
+  id.widget[1] = bind_edge;
+  id.widget[2] = left_edge;
+  id.widget[3] = top_edge;
+  id.index = index;
+  _change_format (mi, &id);
   gtk_widget_show (frame);
   gtk_widget_show (GTK_WIDGET (opts));  
 
   g_print ("%s: exit\n", __func__);
 }
 
-void
-file_selector::add_dialog_extensions (bool)
-{
-  return add_dialog_extensions ();
-}
-
 bool
-file_selector::permission( const char *file ) const
+file_selector::permission( const char *file_in ) const
 {
+  if (!file_in || 0 == strlen (file_in))
+    return false;
+
+  char* file = strdup (file_in);
+
+  if (!file)
+    return false;
+
   if (0 == access( file, F_OK ))	// file exists
-    return (0 == access( file, W_OK ));	// whether we can write to it
+    {
+      bool rv = (0 == access( file, W_OK ));	// whether we can write to it
+      free (file);
+      return rv;
+    }
 
   // check write access to the directory (note that we need execute
   // privileges as well)
@@ -1035,18 +930,21 @@ file_selector::permission( const char *file ) const
 
   *slash = '/';			// restore filename
 
+  free (file);
   return w_ok;
 }
 
 bool
-file_selector::show_message( const pisa_error& oops, bool yes_no ) const
+file_selector::show_message( const pisa_error& oops,
+                             const char *yes,
+                             const char *no ) const
 {
   aleart_dialog dlg;
 
-  if (yes_no)			// binary question
+  if (yes && no)		// binary question
     return (1 == dlg.message_box( GTK_WIDGET( _widget ),
 				  oops.get_error_string(),
-				  _( "  Yes  " ), _( "  No  " ) ));
+				  yes, no ));
 
   dlg.message_box( GTK_WIDGET( _widget ), oops.get_error_string() );
   return true;
@@ -1063,3 +961,177 @@ file_selector::regerror (int code, regex_t *regex) const
 
   delete[] message;
 }
+
+std::string
+file_selector::get_format_name () const
+{
+  return _format[_format_index].name;
+}
+
+iscan::file_format
+file_selector::get_type () const
+{
+  return _format[_format_index].type;
+}
+
+bool
+file_selector::multi_page_mode ()
+{
+  if (iscan::PDF == get_type () || iscan::TIF == get_type ())
+    return _multi_page_mode;
+
+  return false;
+}
+
+bool
+file_selector::has_left_edge_binding ()
+{
+  return _has_left_edge_binding;
+}
+
+gint
+file_selector::get_index (const char* name) const
+{
+  if (!name) return 0;
+
+  int i = 0;
+  while (_format[i].name && (0 != strcmp (name, _format[i].name)))
+    {
+      ++i;
+    }
+  return (_format[i].name ? i : 0);
+}
+
+bool
+file_selector::is_supported (const char* format) const
+{
+  // FIXME! ugly kludge to "check" for support
+  if (!format)
+    {
+      return false;
+    }
+  if (0 == strcmp ("PCX", format))
+    {
+      return iscan::pcxstream::is_usable ();
+    }
+  if (0 == strcmp ("PNM", format))
+    {
+      return iscan::pnmstream::is_usable ();
+    }
+  if (0 == strcmp ("PNG", format))
+    {
+      return iscan::pngstream::is_usable ();
+    }
+  if (0 == strcmp ("JPEG", format))
+    {
+      return iscan::jpegstream::is_usable ();
+    }
+  if (0 == strcmp ("PDF", format))
+    {
+      return iscan::pdfstream::is_usable ();
+    }
+  if (0 == strcmp ("TIFF", format))
+    {
+      return iscan::tiffstream::is_usable ();
+    }
+
+  return false;
+}
+
+#include <sstream>
+#include <iostream>
+bool
+file_selector::possible_overwrite (const std::string& current) const
+{
+  if (!_using_adf || _multi_page_mode)
+    return (0 == access (current.c_str (), F_OK));
+
+  string::size_type dot  = current.rfind (file_opener::ext_sep);
+  string::size_type hash = current.find_last_not_of (file_opener::hash_mark,
+                                                     dot - 1);
+  int digits  = dot - ++hash;
+
+  stringstream ss;
+  ss << current.substr (0, hash)
+     << "([1-9][0-9]{" << digits << ",}|[0-9]{" << digits << "})\\"
+     << current.substr (dot);
+
+  return check_overwrite (ss.str ().c_str ());
+}
+
+int
+file_selector::check_overwrite (const char *regexp_in) const
+{
+  bool found = false;
+
+  if (!regexp_in)
+    return false;
+
+  char* regexp = strdup (regexp_in);
+  if (!regexp)
+    return true;
+
+  char *slash = strrchr( regexp, '/' );
+
+  if (!slash)
+    {
+      free (regexp);
+      return true;
+    }
+
+  *slash = '\0';                // regexp now holds the directory name
+  char dirname[ strlen( regexp )];
+  strcpy( dirname, regexp );
+
+  *slash = '^';                 // re-anchor the regexp
+
+  regex_t *comp_regex = new regex_t;
+  int comp = regcomp( comp_regex, slash, REG_EXTENDED );
+
+  if (0 == comp)
+    {
+      size_t     nsub = comp_regex->re_nsub + 1;
+      regmatch_t match[nsub];
+
+      DIR *dir = opendir( dirname );
+      if (!dir)
+        {
+          regfree( comp_regex );
+          delete comp_regex;
+          free (regexp);
+          return 0;               // file creation failure handles this
+        }
+
+      struct dirent *file = 0;
+      while (!found && (file = readdir( dir )))
+        {
+          int result = regexec( comp_regex, file->d_name, nsub, match, 0 );
+          if (0 == result)
+            {
+              size_t digits = match[1].rm_eo - match[1].rm_so;
+              char num[digits + 1];
+              char *c = num;
+              {
+                char *p = file->d_name + match[1].rm_so;
+                while (0 < digits--)
+                  *c++ = *p++;
+              }
+              int seq_num = atoi( num );
+
+              found = (seq_num >= get_sequence_number());
+            }
+          else
+            if (REG_NOMATCH != result)
+              regerror( comp, comp_regex );
+        }
+      closedir( dir );
+    }
+  else
+    regerror( comp, comp_regex );
+
+  regfree( comp_regex );
+  delete comp_regex;
+  free (regexp);
+
+  return found;
+}
