@@ -1,6 +1,6 @@
 /* 
    SANE EPSON backend
-   Copyright (C) 2001, 2005 SEIKO EPSON Corporation
+   Copyright (C) 2001, 2005, 2008  SEIKO EPSON CORPORATION
 
    Date         Author      Reason
    06/01/2001   N.Sasaki    New
@@ -137,19 +137,12 @@ int gamma_correction::init ( void )
 }
 
 /*------------------------------------------------------------*/
-GtkWidget * gamma_correction::create_window ( GtkWidget * parent )
+GtkWidget * gamma_correction::create_controls ( GtkWidget * parent )
 {
-  GtkWidget	* frame;
   GtkWidget	* vbox;
   GtkWidget	* widget;
 
-  frame = ::gtk_frame_new ( _( "Tone Correction" ) );
-  ::gtk_container_border_width ( GTK_CONTAINER ( frame ), 5 );
-
   vbox = ::gtk_vbox_new ( FALSE, 2 );
-  ::gtk_container_add ( GTK_CONTAINER ( frame ), vbox );
-  ::gtk_widget_show ( vbox );
-
   widget = create_gamma_notebook ( parent );
   ::gtk_box_pack_start ( GTK_BOX ( vbox ), widget, FALSE, FALSE, 0 );
   ::gtk_widget_show ( widget );
@@ -162,7 +155,7 @@ GtkWidget * gamma_correction::create_window ( GtkWidget * parent )
   m_cursor [ PISA_CS_GM_FLEUR  ] = ::gdk_cursor_new ( GDK_FLEUR );
   m_cursor [ PISA_CS_GM_TCROSS ] = ::gdk_cursor_new ( GDK_TCROSS );
 
-  return frame;
+  return vbox;
 }
 
 /*------------------------------------------------------------*/
@@ -185,15 +178,12 @@ int gamma_correction::close_window ( int destroy )
 /*------------------------------------------------------------*/
 void gamma_correction::sensitive ( int is_prev_img )
 {
-  settings	* set;
-  marquee	* marq;
   gboolean	enable_rgb, enable_r_g_b;
 
-  set = & ::g_view_manager->m_set;
-  marq = set->get_current_marquee ( );
+  settings set = g_view_manager->get_settings ();
 
   // grayout
-  switch ( set->imgtype.pixeltype )
+  switch ( set.imgtype.pixeltype )
     {
     case PISA_PT_RGB:
       enable_rgb	= TRUE;
@@ -270,8 +260,7 @@ gint gamma_correction::event ( GtkWidget * widget, GdkEvent * event )
 {
   static int cursor_type = PISA_CS_GM_FLEUR;
   int new_type;
-  GdkEventButton	* bevent;
-  GdkEventMotion	* mevent;
+  GdkEventMotion	* mevent = NULL;
   gint	tx, ty, x, y;
   gint	i, distance, closest_point;
 
@@ -308,7 +297,6 @@ gint gamma_correction::event ( GtkWidget * widget, GdkEvent * event )
       break;
 
     case GDK_BUTTON_PRESS:
-      bevent = ( GdkEventButton * ) event;
       new_type = PISA_CS_GM_TCROSS;
 
       m_leftmost = -1;
@@ -529,11 +517,8 @@ void gamma_correction::update_gamma ( void )
 {
   int i;
   GdkPoint points [ GAMMA_WIDTH ];
-  marquee	* marq;
   GdkGC		* gc;
 
-  marq = ::g_view_manager->m_set.get_current_marquee ( );
-
   if ( m_pixmap [ m_page ] == 0 )
     return;
 
@@ -630,7 +615,7 @@ void gamma_correction::calculate_curve ( void )
       plot_curve ( p1, p2, p3, p4 );
     }
 
-  marq = ::g_view_manager->m_set.get_current_marquee ( );
+  marq = & g_view_manager->get_marquee ();
 
   for ( i = 0; i < 256; i++ )
     marq->gamma_table [ m_page ] [ i ] =
