@@ -1,6 +1,6 @@
 /* 
    SANE EPSON backend
-   Copyright (C) 2001, 2005 SEIKO EPSON Corporation
+   Copyright (C) 2001, 2005, 2009 SEIKO EPSON CORPORATION
 
    Date         Author      Reason
    06/01/2001   N.Sasaki    New
@@ -45,6 +45,7 @@
 static void * handle_libgimp = 0;
 int g_gimpversion = 1;
 
+typedef gchar* lib_gimp_version (void);
 typedef gint lib_gimp_main ( gint argc, gchar * argv [ ] );
 typedef void lib_gimp_quit ( void );
 typedef void lib_gimp_install_procedure ( gchar * name,
@@ -122,6 +123,7 @@ typedef gint lib_gimp_main_2 (  const GimpPlugInInfo_2 *info,
 			     gint argc, gchar * argv [ ] );	
 #endif 
 
+lib_gimp_version * plib_gimp_version;
 lib_gimp_main * plib_gimp_main;
 lib_gimp_quit * plib_gimp_quit;
 lib_gimp_install_procedure * plib_gimp_install_procedure;
@@ -259,6 +261,7 @@ gint pisa_gimp_main ( gint argc, gchar * argv [ ] )
     }
   }
 
+  plib_gimp_version                = ( lib_gimp_version * ) dlsym ( handle_libgimp, "gimp_version" );
   plib_gimp_quit                   = ( lib_gimp_quit * ) dlsym ( handle_libgimp, "gimp_quit" );
   plib_gimp_tile_height            = ( lib_gimp_tile_height * ) dlsym ( handle_libgimp, "gimp_tile_height" );
   plib_gimp_image_new              = ( lib_gimp_image_new * ) dlsym ( handle_libgimp, "gimp_image_new" );
@@ -335,8 +338,8 @@ static void query ( void )
 			      "This function provides access to scanners "
 			      "and other image acquisition devices through "
 			      "the SANE (Scanner Access Now Easy) interface.",
-			      "EPSON AVASYS Corporation",
-			      "SEIKO EPSON Corporation",
+			      "AVASYS CORPORATION",
+			      "SEIKO EPSON CORPORATION",
 			      "2001, 2005",
 			      "<Toolbox>/Xtns/Acquire Image/Scanning (iscan)...",
 			      "RGB, GRAY",
@@ -356,6 +359,20 @@ static void gimp2_query ( void )
   static int		nargs = sizeof ( args ) / sizeof ( args [ 0 ] );
   static int		nreturn_vals = 0;
 
+  char* menu = "<Toolbox>/File/Acquire/Scanning (iscan)...";
+  int maj, min = 0;
+  if (!plib_gimp_version ||
+      2 != sscanf (plib_gimp_version (), "%d.%d.", &maj, &min))
+    {
+      maj = 2;
+      min = 0;
+    }
+  if (2 < maj || (2 == maj && 6 <= min))
+    {
+      // accommodate new menu structure introduced in gimp 2.6
+      menu = "<Image>/File/Create/Acquire/Scanning (iscan)...";
+    }
+
   // Must call this function for supporting gimp-2 
   plib_gimp_install_procedure_2 (
 			      "Image Scan! for Linux",
@@ -363,11 +380,11 @@ static void gimp2_query ( void )
 			      "This function provides access to scanners "
 			      "and other image acquisition devices through "
 			      "the SANE (Scanner Access Now Easy) interface.",
-			      "EPSON AVASYS Corporation",
-			      "SEIKO EPSON Corporation",
-			      "2005",
-			      "<Toolbox>/Xtns/Acquire Image/Scanning (iscan)...",
-			      "RGB, GRAY",
+			      "AVASYS CORPORATION",
+			      "SEIKO EPSON CORPORATION",
+			      "2009",
+			      menu,
+			      "",
 			      GIMP_EXTENSION,
 			      nargs, nreturn_vals,
 			      args, return_vals );
