@@ -1,6 +1,6 @@
 /* 
    SANE EPSON backend
-   Copyright (C) 2001 SEIKO EPSON Corporation
+   Copyright (C) 2001 SEIKO EPSON CORPORATION
 
    Date         Author      Reason
    06/01/2001   N.Sasaki    New
@@ -35,20 +35,15 @@
 
 class image_controls
 {
+  float brightness_default;
+  float contrast_default;
+
  public:
 
   // operation
-  GtkWidget *	create_window ( GtkWidget * parent );
   void	sensitive ( int is_prev_img );
 
- private:
-
-  // operation
-
   GtkWidget	* create_controls ( void );
-
-  // attribute
-
 };
 
 #endif // ___PISA_IMAGE_CONTROLS_H
