@@ -1,5 +1,5 @@
 /* pisa_preview_window.cc
-   Copyright (C) 2001, 2004, 2005 SEIKO EPSON Corporation
+   Copyright (C) 2001, 2004, 2005, 2008, 2009  SEIKO EPSON CORPORATION
 
    This file is part of the `iscan' program.
 
@@ -169,16 +169,11 @@ int preview_window::auto_exposure ( void )
   _rectL		region;
   marquee		tmp_marq, * marq;
   int			i, marq_num;
-  settings		* set;
 
-  scan_manager	* scan_mgr;
-  const scanner_info	* scan_info;
-
-  scan_mgr = ::g_view_manager->get_scan_manager ( );
-  scan_info = scan_mgr->get_scanner_info ( );
+  scan_manager	*scan_mgr = g_view_manager->get_scan_manager ();
 
   if ( m_is_prev == 0 ||
-       ::g_view_manager->m_set.imgtype.pixeltype == PISA_PT_BW )
+       g_view_manager->get_settings ().imgtype.pixeltype == PISA_PT_BW )
     {
       return PISA_ERR_SUCCESS;
     }
@@ -188,9 +183,8 @@ int preview_window::auto_exposure ( void )
   info.m_height	= m_img_height;
   info.m_rowbytes = g_prev_max_x * 3;
 
-  set = & ::g_view_manager->m_set;
-  marq_num = set->get_marquee_size ( );
-  marq = set->get_current_marquee ( );
+  marq_num = g_view_manager->get_marquee_size ();
+  marq = & g_view_manager->get_marquee ();
 
   if ( marq_num < 2 )
     {
@@ -211,13 +205,13 @@ int preview_window::auto_exposure ( void )
       region.bottom	= pt_rb.y;
     }
 
-  iscan::auto_expose (set->option,
-		      set->film,
-		      & info,
-		      & region,
-		      & tmp_marq,
-		      set->imgtype.ae_option != PISA_AE_DOC,
-		      scan_info->dumb);
+  settings set = g_view_manager->get_settings ();
+  scan_manager *sm = g_view_manager->get_scan_manager ();
+  iscan::auto_expose (sm->get_scan_source (),
+                      sm->get_film_type (),
+                      info, region, tmp_marq,
+		      set.imgtype.ae_option != PISA_AE_DOC,
+		      !scan_mgr->has_zoom ());
 
   marq->gamma		= tmp_marq.gamma;
   marq->highlight	= tmp_marq.highlight;
@@ -242,7 +236,6 @@ int preview_window::update_img ( bool left )
 {
   int			i;
   pisa_image_info	img_info;
-  settings		* set;
   marquee		* marq;
   double                delta;
   long                  curr_width, adj_width, resolution;
@@ -253,10 +246,10 @@ int preview_window::update_img ( bool left )
       return PISA_ERR_SUCCESS;
     }
 
-  set        = & g_view_manager->m_set;
-  marq       = set->get_current_marquee ( );
+  settings set = g_view_manager->get_settings ();
+  marq = & g_view_manager->get_marquee ();
 
-  resolution = ( set->resolution * (marq->scale)  ) / 100;
+  resolution = (set.resolution * (marq->scale)) / 100;
   curr_width = ::inch2pixel ( marq->area.x, resolution );
   adj_width  = ::inch2width ( marq->area.x, resolution );
   delta      = marq->area.x - (marq->area.x) * adj_width /curr_width ;
@@ -281,25 +274,25 @@ int preview_window::update_img ( bool left )
 
   ::tool_lut ( & img_info, & marq->lut );
 
-  switch ( set->imgtype.pixeltype )
+  switch (set.imgtype.pixeltype)
     {
     case PISA_PT_RGB:
-      ::tool_matrix ( & img_info, marq->saturation, set->coef );
-      if (set->usm)
+      tool_matrix (&img_info, marq->saturation, set.coef);
+      if (set.usm)
 	tool_usm (img_info);
       break;
 
     case PISA_PT_GRAY:
-      ::build_gray ( & img_info, set->imgtype.dropout );
-      if (set->usm)
+      build_gray (&img_info, set.imgtype.dropout);
+      if (set.usm)
 	tool_usm (img_info);
       break;
 
     case PISA_PT_BW:
-      ::build_bw ( & img_info,
-		   set->imgtype.dropout,
-		   set->imgtype.halftone,
-		   marq->threshold );
+      build_bw (&img_info,
+		set.imgtype.dropout,
+		set.imgtype.halftone,
+		marq->threshold);
       break;
     }
 
@@ -314,14 +307,14 @@ int preview_window::update_img ( bool left )
   return PISA_ERR_SUCCESS;
 }
 
-/*--------------------------------------------------------------*/
-void preview_window::start_preview ( pisa_preview_type type )
+void
+preview_window::start_preview (bool zooming)
 {
-  scan_manager		* scan_mgr;
-  scan_parameter	param;
+  scan_manager		* scan_mgr =
+    g_view_manager->get_scan_manager ( );
   int			width, height;
   int			cancel;
-  
+
   cancel = 0;
 
   m_is_prev = 0;
@@ -334,24 +327,47 @@ void preview_window::start_preview ( pisa_preview_type type )
     (static_cast <main_window *> (g_view_manager
 				  ->get_window_cls (ID_WINDOW_MAIN))
      ->get_widget());
-  feedback.set_text (progress_window::WARMING_UP);
+  if (scan_mgr->push_button_needs_polling ())
+  {
+    feedback.set_text (progress_window::WAITING);
+  }
+  else
+  {
+    feedback.set_text (progress_window::WARMING_UP);
+  }
   feedback.show ();
 
   while ( ::gtk_events_pending ( ) )
     ::gtk_main_iteration ( );
 
   // preview
-  scan_mgr = g_view_manager->get_scan_manager ( );
-  set_preview_param ( & param, type );
+  set_preview_param (zooming);
 
-  reset_settings ( type );
+  reset_settings (zooming);
   ::g_view_manager->sensitive ( );
 
   try
     {
-      scan_mgr->init_scan (&param, &width, &height);
+      if (scan_mgr->push_button_needs_polling ())
+      {
+        long usec = scan_mgr->get_polling_time ();
+        while (!scan_mgr->is_button_pressed ()
+               && !feedback.is_cancelled ()) {
+          g_view_manager->microsleep (usec);
+          while (::gtk_events_pending ()) {
+            ::gtk_main_iteration ();
+          }
+        }
+        if (feedback.is_cancelled ())
+        {
+          return;
+        }
+        scan_mgr->disable_wait_for_button();
+      }
 
-      if ( type == PISA_PREV_WHOLE )
+      scan_mgr->init_preview (&width, &height);
+
+      if (!zooming)
 	{
 	  change_max_scan_area ( width, height );
 	  ::g_view_manager->sensitive ( );
@@ -393,6 +409,7 @@ void preview_window::start_preview ( pisa_preview_type type )
 	  while ( ::gtk_events_pending ( ) )
 	    ::gtk_main_iteration ( );
 	}
+      scan_mgr->acquire_image (0, 1, 1, cancel);
       feedback.set_progress (height, height);
       cancel = feedback.is_cancelled ();
       if (cancel)
@@ -423,9 +440,12 @@ void preview_window::start_preview ( pisa_preview_type type )
       main_window * main_cls;
       aleart_dialog aleart_dlg;
 
-      main_cls = ( main_window * ) ::g_view_manager->get_window_cls ( ID_WINDOW_MAIN );
-      aleart_dlg.message_box ( main_cls->get_widget ( ),
-			      err.get_error_string ( ) );
+      if ( PISA_STATUS_CANCELLED != err.get_error_id() )
+        {
+          main_cls = ( main_window * ) ::g_view_manager->get_window_cls ( ID_WINDOW_MAIN );
+          aleart_dlg.message_box ( main_cls->get_widget ( ),
+			           err.get_error_string ( ) );
+        }
 
       delete_marquee ( );
 
@@ -436,7 +456,10 @@ void preview_window::start_preview ( pisa_preview_type type )
       m_allocate_height = 0;
     }
 
-  scan_mgr->finish_acquire ( );
+  revert_resolution ();
+
+  scan_mgr->release_memory ();
+  scan_mgr->finish_acquire (true);
 
   ::g_view_manager->sensitive ( );
 
@@ -577,8 +600,8 @@ long preview_window::get_preview_resolution ( const _rectD * img_rect )
     }
   else
     {
-      pt_scan_size.x = ::g_view_manager->m_set.max_area [ 0 ];
-      pt_scan_size.y = ::g_view_manager->m_set.max_area [ 1 ];
+      pt_scan_size.x = g_view_manager->get_settings ().max_area [ 0 ];
+      pt_scan_size.y = g_view_manager->get_settings ().max_area [ 1 ];
 
       m_img_rect.left	= 0.0;
       m_img_rect.top	= 0.0;
@@ -638,10 +661,10 @@ void preview_window::change_max_scan_area ( long width, long height )
   if ( m_img_width < width || m_img_height < height )
     return;
 
-  marq = ::g_view_manager->m_set.get_current_marquee ( );
-  
-  pt_scan_size.x = ::g_view_manager->m_set.max_area [ 0 ];
-  pt_scan_size.y = ::g_view_manager->m_set.max_area [ 1 ];
+  marq = & g_view_manager->get_marquee ();
+
+  pt_scan_size.x = g_view_manager->get_settings ().max_area [ 0 ];
+  pt_scan_size.y = g_view_manager->get_settings ().max_area [ 1 ];
 
   m_img_rect.left	= 0.0;
   m_img_rect.top	= 0.0;
@@ -759,7 +782,6 @@ void preview_window::clear_image ( void )
 /*--------------------------------------------------------------*/
 void preview_window::draw_marquee ( void )
 {
-  settings	* set;
   long		i, marq_num;
   _pointL	pt_lefttop, pt_rightbottom;
 
@@ -771,9 +793,7 @@ void preview_window::draw_marquee ( void )
 				     GDK_CAP_BUTT, GDK_JOIN_MITER );
     }
 
-  set = & ::g_view_manager->m_set;
-
-  marq_num = set->get_marquee_size ( );
+  marq_num = g_view_manager->get_settings ().get_marquee_size ( );
 
   if ( marq_num < 2 || m_on_preview )
     return;
@@ -787,41 +807,37 @@ void preview_window::draw_marquee ( void )
 }
 
 /*--------------------------------------------------------------*/
-void preview_window::reset_settings ( pisa_preview_type type )
+void
+preview_window::reset_settings (bool zooming)
 {
-  settings	* set;
   long		i, n, m, marq_num;
   marquee	* marq;
   gamma_correction * gamma_cls;
   scan_manager	* scan_mgr;
-  const scanner_info	* scan_info;
 
-  gamma_cls = ( gamma_correction * ) ::g_view_manager->get_window_cls ( ID_WINDOW_GAMMA );
+  gamma_cls = g_view_manager->get_gamma_correction ();
 
-  scan_mgr = ::g_view_manager->get_scan_manager ( );
-  scan_info = scan_mgr->get_scanner_info ( );
+  scan_mgr = g_view_manager->get_scan_manager ();
+  marq_num = g_view_manager->get_marquee_size ();
 
-  set = & ::g_view_manager->m_set;
-  marq_num = set->get_marquee_size ( );
-
-  if ( type == PISA_PREV_WHOLE )
+  if (!zooming)
     {
       while ( 1 < marq_num )
 	{
-	  set->del_marquee ( marq_num - 1 );
-	  marq_num = set->get_marquee_size ( );
+	  g_view_manager->del_marquee (marq_num - 1);
+	  marq_num = g_view_manager->get_marquee_size ();
 	}
 
       gamma_cls->reset ( 1 );
     }
 
-  marq_num = set->get_marquee_size ( );
+  marq_num = g_view_manager->get_marquee_size ();
 
   for ( i = 0; i < marq_num; i++ )
     {
-      marq = set->get_marquee ( i );
+      marq = & g_view_manager->get_marquee (i);
 
-      if ( type == PISA_PREV_WHOLE )
+      if (!zooming)
 	{
 	  marq->gamma		= ( long ) ( 100 * DEFGAMMA );
 	  marq->highlight	= DEFHIGHLIGHT;
@@ -843,113 +859,71 @@ void preview_window::reset_settings ( pisa_preview_type type )
 	  marq->grayl [ n ]		= 0.0;
 	}
       
-      iscan::build_LUT ( set, marq, scan_info->dumb );
+      iscan::build_LUT (scan_mgr->get_scan_source (),
+                        scan_mgr->get_film_type (),
+                        g_view_manager->get_settings (),
+                        *marq, !scan_mgr->has_zoom ());
     }
 }
 
-/*--------------------------------------------------------------*/
-int preview_window::set_preview_param ( scan_parameter * param,
-					pisa_preview_type type )
+int
+preview_window::set_preview_param (bool zooming)
 {
-  settings	* set;
-  marquee	* cur_marq, * whole_marq;
-  int		i;
-  _rectD	img_rect;
-  long		resolution;
+  settings set = g_view_manager->get_settings ();
+  marquee marq;
 
-  ::memset ( param, 0, sizeof ( scan_parameter ) );
-
-  set = & ::g_view_manager->m_set;
-  cur_marq = set->get_current_marquee ( );
-  
-  // document source
-  param->option		= set->option;
-  param->film		= set->film;
-
-  // image type
-  param->pixeltype	= PISA_PT_RGB;
-  param->bitdepth	= PISA_BD_8;
-  param->scanspeed	= PISA_SS_DRAFT;
-  param->dropout	= PISA_DO_NONE;
-  param->monoopt	= PISA_MO_NONE;
-  param->halftone	= PISA_HT_NONE;
-
-  // gamma table
-  for ( i = 0; i < 256; i++ )
+  if (!zooming)
     {
-      param->gamma.gamma_r [ i ]	= i;
-      param->gamma.gamma_g [ i ]	= i;
-      param->gamma.gamma_b [ i ]	= i;
-    }
-
-  // profile matrix
-  param->coef [ 0 ]	= 1.0;
-  param->coef [ 1 ]	= 0.0;
-  param->coef [ 2 ]	= 0.0;
-  param->coef [ 3 ]	= 0.0;
-  param->coef [ 4 ]	= 1.0;
-  param->coef [ 5 ]	= 0.0;
-  param->coef [ 6 ]	= 0.0;
-  param->coef [ 7 ]	= 0.0;
-  param->coef [ 8 ]	= 1.0;
-
-  // threshold
-  param->threshold	= 0;
-
-  // speed
-  param->speed		= 1;
-
-  // focus
-  param->focus		= cur_marq->focus;
-
-  // unsharp mask
-  param->usm		= 0;	// off
-
-  // de-screening;
-  param->de_screening	= 0;
-
-  // offset area
-  if ( type == PISA_PREV_WHOLE )
-    {
-      param->offset [ 0 ]	= 0.0;
-      param->offset [ 1 ]	= 0.0;
-      param->area [ 0 ]		= set->max_area [ 0 ];
-      param->area [ 1 ]		= set->max_area [ 1 ];
+      marq.offset.x	= 0.0;
+      marq.offset.y	= 0.0;
+      marq.area.x	= set.max_area[0];
+      marq.area.y	= set.max_area[1];
     }
   else
     {
       float rate = 0.1f;
       _pointD pt_offset, pt_area, pt_max;
 
-      whole_marq = set->get_marquee ( 0 );
+      marquee cur_marq = g_view_manager->get_marquee ();
 
-      pt_offset	= cur_marq->offset;
-      pt_area	= cur_marq->area;
+      marq.focus = cur_marq.focus;
 
-      pt_max.x = whole_marq->area.x;
-      pt_max.y = whole_marq->area.y;
+      pt_offset	= cur_marq.offset;
+      pt_area	= cur_marq.area;
 
-      zoom_boundary ( & pt_offset, & pt_area, pt_max, rate );
-      
-      param->offset [ 0 ]	= pt_offset.x;
-      param->offset [ 1 ]	= pt_offset.y;
-      param->area [ 0 ]		= pt_area.x;
-      param->area [ 1 ]		= pt_area.y;      
+      marquee whole_marq = g_view_manager->get_marquee (0);
+
+      pt_max.x = whole_marq.area.x;
+      pt_max.y = whole_marq.area.y;
+
+      zoom_boundary (pt_offset, pt_area, pt_max, rate);
+
+      marq.offset.x	= pt_offset.x;
+      marq.offset.y	= pt_offset.y;
+      marq.area.x	= pt_area.x;
+      marq.area.y	= pt_area.y;
     }
 
-  // resolution scale
-  img_rect.left		= param->offset [ 0 ];
-  img_rect.top		= param->offset [ 1 ];
-  img_rect.right	= param->offset [ 0 ] + param->area [ 0 ];
-  img_rect.bottom	= param->offset [ 1 ] + param->area [ 1 ];
-  resolution = get_preview_resolution ( & img_rect );
+  _rectD	img_rect;
+  img_rect.left		= marq.offset.x;
+  img_rect.top		= marq.offset.y;
+  img_rect.right	= marq.offset.x + marq.area.x;
+  img_rect.bottom	= marq.offset.y + marq.area.y;
 
-  param->resolution [ 0 ]	= resolution;
-  param->resolution [ 1 ]	= resolution;
-  param->zoom [ 0 ]		= 100;
-  param->zoom [ 1 ]		= 100;
+  long resolution = get_preview_resolution (&img_rect);
 
-  return PISA_ERR_SUCCESS;
+  pisa_error_id err = (g_view_manager->get_scan_manager ()
+		       ->set_scan_parameters (set, marq, resolution));
+
+  return err;
+}
+
+void
+preview_window::revert_resolution ( void )
+{
+  long set_res = g_view_manager->get_settings ().resolution;
+
+  g_view_manager->set_resolution (set_res);
 }
 
 /*------------------------------------------------------------*/
@@ -961,46 +935,44 @@ void preview_window::tool_usm (const pisa_image_info& info)
 	  info.m_img, info.m_height * info.m_rowbytes);
 }
 
-/*--------------------------------------------------------------*/
-void preview_window::zoom_boundary ( _pointD * pt_offset,
-				     _pointD * pt_area,
-				     _pointD & pt_max,
-				     float rate )
+void
+preview_window::zoom_boundary (_pointD& pt_offset, _pointD& pt_area,
+			       const _pointD& pt_max, float rate) const
 {
   _pointD pt_result_offset, pt_result_area;
   double bound_size;
 
-  if ( pt_area->x < pt_area->y )
-    bound_size = pt_area->y * rate;
+  if (pt_area.x < pt_area.y)
+    bound_size = pt_area.y * rate;
   else
-    bound_size = pt_area->x * rate;
+    bound_size = pt_area.x * rate;
 
-  bound_size = ( bound_size == 0.0 ) ? 0.01 : bound_size;
+  bound_size = (0.0 == bound_size) ? 0.01 : bound_size;
 
   // left
-  pt_result_offset.x = pt_offset->x - bound_size;
-  if ( pt_result_offset.x < 0.0 )
+  pt_result_offset.x = pt_offset.x - bound_size;
+  if (0.0 > pt_result_offset.x)
     pt_result_offset.x = 0.0;
 
   // top
-  pt_result_offset.y = pt_offset->y - bound_size;
-  if ( pt_result_offset.y < 0.0 )
+  pt_result_offset.y = pt_offset.y - bound_size;
+  if (0.0 > pt_result_offset.y)
     pt_result_offset.y = 0.0;
 
   // right
-  pt_result_area.x = ( pt_offset->x - pt_result_offset.x ) +
-    pt_area->x + bound_size;
-  if ( pt_max.x < pt_result_offset.x + pt_result_area.x )
+  pt_result_area.x = ((pt_offset.x - pt_result_offset.x) +
+		      pt_area.x + bound_size);
+  if (pt_max.x < pt_result_offset.x + pt_result_area.x)
     pt_result_area.x = pt_max.x - pt_result_offset.x;
 
   // bottom
-  pt_result_area.y = ( pt_offset->y - pt_result_offset.y ) +
-    pt_area->y + bound_size;
-  if ( pt_max.y < pt_result_offset.y + pt_result_area.y )
+  pt_result_area.y = ((pt_offset.y - pt_result_offset.y) +
+		      pt_area.y + bound_size);
+  if (pt_max.y < pt_result_offset.y + pt_result_area.y)
     pt_result_area.y = pt_max.y - pt_result_offset.y;
 
-  * pt_offset = pt_result_offset;
-  * pt_area   = pt_result_area;
+  pt_offset = pt_result_offset;
+  pt_area   = pt_result_area;
 }
 
 /*--------------------------------------------------------------*/
@@ -1045,7 +1017,7 @@ int preview_window::set_mouse_cursor ( int x, int y )
       return FALSE;
     }
 
-  marq_num = ::g_view_manager->m_set.get_marquee_size ( );
+  marq_num = g_view_manager->get_settings ().get_marquee_size ( );
 
   if ( marq_num < 2 )
     {
@@ -1200,7 +1172,7 @@ void preview_window::modify_max_val ( void )
   marquee	* whole_marq;
   _pointD	pt_min_D, pt_max_D;
 
-  whole_marq = ::g_view_manager->m_set.get_marquee ( 0 );
+  whole_marq = & g_view_manager->get_marquee (0);
   m_max_img_rect.top	= 0.0;
   m_max_img_rect.left	= 0.0;
   m_max_img_rect.right	= whole_marq->area.x;
@@ -1237,10 +1209,10 @@ gint preview_window::mouse_down ( GdkEvent * event )
   
   m_pt_old = m_pt_begin;
 
-  id_cur_marquee = ::g_view_manager->m_set.get_marquee_size ( ) - 1;
+  id_cur_marquee = g_view_manager->get_marquee_size () - 1;
   get_marquee_point ( id_cur_marquee, & pt_lefttop, & pt_rightbottom );
 
-  cur_marq = ::g_view_manager->m_set.get_current_marquee ( );
+  cur_marq = & g_view_manager->get_marquee ();
 
   m_pt_save_offset	= cur_marq->offset;
   m_pt_save_area	= cur_marq->area;
@@ -1409,7 +1381,7 @@ gint preview_window::mouse_move ( GdkEvent * event )
 /*--------------------------------------------------------------*/
 gint preview_window::mouse_up ( GdkEvent * event )
 {
-  int min_check, move;
+  int min_check;
   _pointL pt_mouse;
   long	marq_num;
 
@@ -1419,10 +1391,8 @@ gint preview_window::mouse_up ( GdkEvent * event )
   pt_mouse.x = ( int ) event->button.x;
   pt_mouse.y = ( int ) event->button.y;
 
-  move = ( m_pt_begin.x != pt_mouse.x ) || ( m_pt_begin.y != pt_mouse.y );
-
   // clear previous marquee
-  marq_num = ::g_view_manager->m_set.get_marquee_size ( );
+  marq_num = g_view_manager->get_marquee_size ();
   if ( 1 < marq_num )
     draw_rect ( m_pt_old_lt, m_pt_old_rb );
 
@@ -1470,9 +1440,6 @@ int preview_window::create_marquee ( const _pointL & pt_lt, const _pointL & pt_r
   _pointL	pt_lefttop_L, pt_rightbottom_L;
   _pointD	pt_lefttop_D, pt_rightbottom_D, pt_area_D;
   marquee	* new_marq, * whole_marq;
-  settings	* set;
-
-  set = & ::g_view_manager->m_set;
 
   pt_lefttop_L		= pt_lt;
   pt_rightbottom_L	= pt_rb;
@@ -1484,14 +1451,14 @@ int preview_window::create_marquee ( const _pointL & pt_lt, const _pointL & pt_r
 
   check_max_size ( & pt_lefttop_D, & pt_area_D, 0 );
 
-  whole_marq = set->get_marquee ( 0 );
+  whole_marq = & g_view_manager->get_marquee (0);
   new_marq = new marquee;
   * new_marq = * whole_marq;
 
   new_marq->offset	= pt_lefttop_D;
   new_marq->area	= pt_area_D;
 
-  set->add_marquee ( & new_marq );
+  g_view_manager->add_marquee (&new_marq);
 
   ::g_view_manager->sensitive ( );
 
@@ -1503,19 +1470,16 @@ int preview_window::delete_marquee ( void )
 {
   long		num_marq;
   int		i, j;
-  settings	* set;
   marquee	* cur_marq, * whole_marq;
 
-  set = & ::g_view_manager->m_set;
-
-  num_marq = set->get_marquee_size ( );
+  num_marq = g_view_manager->get_marquee_size ();
 
   // no marquee
   if ( num_marq < 2 )
     return PISA_ERR_SUCCESS;
 
-  whole_marq = set->get_marquee ( 0 );
-  cur_marq = set->get_current_marquee ( );
+  whole_marq = & g_view_manager->get_marquee (0);
+  cur_marq = & g_view_manager->get_marquee ();
 
   whole_marq->gamma		= cur_marq->gamma;
   whole_marq->highlight		= cur_marq->highlight;
@@ -1547,7 +1511,7 @@ int preview_window::delete_marquee ( void )
       whole_marq->lut.gamma_b [ i ]	= cur_marq->lut.gamma_b [ i ];
     }
 
-  set->del_current_marquee ( );
+  g_view_manager->del_marquee ();
 
   ::g_view_manager->sensitive ( );
   update_img ( );
@@ -1574,7 +1538,7 @@ void preview_window::move_marquee ( const _pointL & pt_lt,
 
   check_max_size ( & pt_offset, & pt_area, 1 );
 
-  cur_marq = ::g_view_manager->m_set.get_current_marquee ( );
+  cur_marq = & g_view_manager->get_marquee ();
 
   cur_marq->offset	= pt_offset;
 
@@ -1602,7 +1566,7 @@ void preview_window::resize_marquee ( const _pointL & pt_lt,
 
   check_max_size ( & pt_offset, & pt_area, 0 );
 
-  cur_marq = ::g_view_manager->m_set.get_current_marquee ( );
+  cur_marq = & g_view_manager->get_marquee ();
 
   cur_marq->offset	= pt_offset;
   cur_marq->area	= pt_area;
@@ -1783,7 +1747,7 @@ int preview_window::get_marquee_point ( long i, _pointL * pt_lt, _pointL * pt_rb
   marquee * marq;
   _pointD pt_tmp_lt, pt_tmp_rb;
 
-  if ( 0 == ( marq = ::g_view_manager->m_set.get_marquee ( i ) ) )
+  if (0 == (marq = & g_view_manager->get_marquee (i)))
     return 0;
 
   pt_tmp_lt = marq->offset;
