@@ -1,6 +1,6 @@
 /* 
    SANE EPSON backend
-   Copyright (C) 2001, 2005 SEIKO EPSON Corporation
+   Copyright (C) 2001, 2005, 2008, 2009  SEIKO EPSON CORPORATION
 
    Date         Author      Reason
    06/01/2001   N.Sasaki    New
@@ -36,7 +36,6 @@
 #include <new>
 using std::bad_alloc;
 
-#include "filter.hh"
 
 /*------------------------------------------------------------*/
 #include "pisa_scan_manager.h"
@@ -44,6 +43,9 @@ using std::bad_alloc;
 #include "pisa_scan_tool.h"
 #include "pisa_change_unit.h"
 
+/*------------------------------------------------------------*/
+#define SCALING_INPUT_WIDTH_LIMIT 21840
+
 /*------------------------------------------------------------*/
 void scan_manager::open_device ( char * name )
 {
@@ -56,56 +58,64 @@ void scan_manager::open_device ( char * name )
   m_moire_img_info  = 0;
   m_sharp_img_info  = 0;
 
-  m_scanner_info.name = 0;
-  sane_scan::open_device ( & m_scanner_info, name );
+  _has_prev_img = false;
+
+  sane_scan::open_device (name);
+  _max_descreen_resolution = get_max_resolution (has_zoom () ? 800 : 600);
 }
 
 void
-scan_manager::init_scan (scan_parameter *param, int *width, int *height,
-			 bool reset_params)
+scan_manager::set_scan_parameters (bool is_preview)
 {
-  if (reset_params)
+  for (int i = 0; i < 2; ++i)
     {
-      for (int i = 0; i < 2; ++i)
-	{
-	  adjust_scan_param (&param->resolution[i], &param->zoom[i]);
-	}
-
-      init_img_process_info (param);
-      sane_scan::set_scan_parameter (param);
+      adjust_scan_param (&_resolution[i], &_zoom[i]);
     }
+  init_img_process_info ();
+  update_settings (is_preview);
+}
 
-  if (area_is_too_large (param))
+void
+scan_manager::start_scan (int *width, int *height)
+{
+  sane_scan::start_scan (width, height);
+  modify_img_info (width, height);
+  create_img_cls ();
+}
+
+void
+scan_manager::init_preview (int *width, int *height)
+{
+  set_scan_parameters (true);
+  start_scan (width, height);
+}
+
+void
+scan_manager::init_scan (int *width, int *height, bool reset_params)
+{
+  if (reset_params) set_scan_parameters ();
+
+  if (area_is_too_large ())
     {
       if (getenv ("ISCAN_DEBUG"))
 	{
 	  fprintf (stderr, "Backend will try to scan with these settings\n");
 	  fprintf (stderr, "Resolution (main,sub): %ld,%ld\n",
-		   param->resolution[0], param->resolution[1]);
+		   _resolution[0], _resolution[1]);
 	  fprintf (stderr, "Zoom       (main,sub): %ld,%ld\n",
-		   param->zoom[0], param->zoom[1]);
+		   _zoom[0], _zoom[1]);
 	  fprintf (stderr, "Offset     (main,sub): %ld,%ld\n",
-		   inch2pixel (param->offset[0], param->resolution[0],
-			       param->zoom[0]),
-		   inch2pixel (param->offset[1], param->resolution[1],
-			       param->zoom[1]));
+		   inch2pixel (_offset[0], _resolution[0], _zoom[0]),
+		   inch2pixel (_offset[1], _resolution[1], _zoom[1]));
 	  fprintf (stderr, "Area       (main,sub): %ld,%ld\n",
-		   inch2pixel (true,
-			       param->area[0], param->resolution[0],
-			       param->zoom[0]),
-		   inch2pixel (false,
-			       param->area[1], param->resolution[1],
-			       param->zoom[1]));
+		   inch2pixel (true , _area[0], _resolution[0], _zoom[0]),
+		   inch2pixel (false, _area[1], _resolution[1], _zoom[1]));
 	}
 
       throw pisa_error (PISA_ERR_AREALARGE);
     }
 
-  sane_scan::start_scan ( width, height );
-
-  modify_img_info ( width, height );
-
-  create_img_cls ( );
+  start_scan (width, height);
 }
 
 /*------------------------------------------------------------*/
@@ -127,9 +137,27 @@ void scan_manager::acquire_image ( unsigned char * img,
 	in_rowbytes = m_resize_info.in_rowbytes;
 
       if ( cancel || img == 0 )
-	{
-	  sane_scan::acquire_image ( 0, 0, 1, 1 );
-	}
+        {
+          if (0 < row_bytes)
+            {
+              // Run kludge to trigger a SANE_STATUS_EOF return value
+              // on sane_read() in sane_scan::acquire_image(), as per
+              // SANE specification requirements.
+              // Fixes problems when used with `net` backend.
+
+              unsigned char b[512];
+              SANE_Status s = SANE_STATUS_GOOD;
+              while (SANE_STATUS_GOOD == s)
+                {
+                  s = sane_scan::acquire_image (b, sizeof (b), 1, cancel);
+                }
+              return;
+            }
+          else
+            {
+              sane_scan::acquire_image (0, 0, height, cancel);
+            }
+        }
       else
 	{
 	  in_line = get_send_in_line ( 1 );
@@ -169,24 +197,16 @@ void scan_manager::acquire_image ( unsigned char * img,
     sane_scan::acquire_image ( img, row_bytes, height, cancel );
 }
 
-/*------------------------------------------------------------*/
-void
-scan_manager::finish_acquire ( void )
-{
-  sane_cancel (m_hdevice);
-  release_memory ();
-}
-
-/*------------------------------------------------------------*/
-int scan_manager::init_img_process_info ( scan_parameter * param )
+int
+scan_manager::init_img_process_info ()
 {
   // set flag
-  if ( m_scanner_info.support_zoom == 0 )
+  if (!has_zoom ())
     m_resize = 1;
   else
     m_resize = 0;
 
-  if ( param->de_screening )	// color or B/W document
+  if (_de_screening && !get_size_check()) // color or B/W document
     {
       m_moire = 1;
       m_resize = 0;
@@ -194,71 +214,73 @@ int scan_manager::init_img_process_info ( scan_parameter * param )
   else
     m_moire = 0;
 
-  m_sharp = param->usm;
+  m_sharp = _usm;
 
   if ( m_resize )
-    init_zoom ( & m_resize_info, param );
+    init_zoom (&m_resize_info);
 
   if ( m_moire )
-    init_moire ( & m_moire_info, param );
+    init_moire (&m_moire_info);
 
   if ( m_sharp )
-    init_sharp ( & m_sharp_info, param );
+    init_sharp (&m_sharp_info);
 
   return PISA_ERR_PARAMETER;
 }
 
-/*------------------------------------------------------------*/
-int scan_manager::init_zoom ( resize_img_info * info, scan_parameter * param )
+int
+scan_manager::init_zoom (resize_img_info *info)
 {
   long act_res[2];
 
-  info->resolution = param->resolution[0];
+  info->resolution = _resolution[0];
 
   for (int i = 0; i < 2; ++i)
     {
-      act_res[i] = param->resolution[i] * param->zoom[i] / 100;
+      act_res[i] = _resolution[i] * _zoom[i] / 100;
     }
 
-  info->out_width	= ::inch2width ( param->area [ 0 ],
-					 act_res [ 0 ] );
-  info->out_height	= ::inch2height ( param->area [ 1 ],
-					 act_res [ 1 ] );
-  info->out_rowbytes	= ::calc_rowbytes ( info->out_width,
-					    static_cast <pisa_pixel_type>
-					    ( param->pixeltype ) );
+  info->out_width	= ::inch2width  (_area[0], act_res[0]);
+  info->out_height	= ::inch2height (_area[1], act_res[1]);
+  info->out_rowbytes	= calc_rowbytes (info->out_width,
+					 static_cast <pisa_pixel_type>
+					 (_pixeltype));
 
   for (int i = 0; i < 2; ++i)
     {
-      param->resolution[i] = get_valid_resolution (act_res[i]);
-      if (!param->resolution[i])
-	{
-	  param->resolution[i] = m_scanner_info.max_resolution;
-	}
-      param->zoom[i] = 100;
+      _resolution[i] = act_res[i];
+      _zoom[i] = 100;
     }
 
-  if (   act_res[0] == param->resolution[0]
-      && act_res[1] == param->resolution[1])
+  try
+    {
+      get_valid_resolution (&_resolution[0], &_resolution[1], true);
+    }
+  catch (const pisa_error& e)
+    {
+      if (pisa_error (PISA_STATUS_UNSUPPORTED) != e)
+	throw (e);
+    }
+
+  if (   act_res[0] == _resolution[0]
+      && act_res[1] == _resolution[1])
     {
       m_resize = 0;
       return PISA_ERR_SUCCESS;
     }
 
-  info->in_width	= ::inch2width ( param->area [ 0 ],
-					 param->resolution [ 0 ] );
-  info->in_height	= ::inch2height ( param->area [ 1 ],
-					 param->resolution [ 1 ] );
-  info->in_rowbytes	= ::calc_rowbytes ( info->in_width,
-					    static_cast <pisa_pixel_type>
-					    ( param->pixeltype ) );
+  info->in_width	= ::inch2width  (_area[0], _resolution[0]);
+  info->in_height	= ::inch2height (_area[1], _resolution[1]);
+  info->in_rowbytes	= calc_rowbytes (info->in_width,
+					 static_cast <pisa_pixel_type>
+					 (_pixeltype));
   
-  info->bits_per_pixel	= ::calc_bitperpix ( static_cast <pisa_pixel_type>
-					    ( param->pixeltype ),
-					     static_cast <pisa_bitdepth_type> 
-					     ( param->bitdepth ) );
+  info->bits_per_pixel	= calc_bitperpix (static_cast <pisa_pixel_type>
+					  (_pixeltype),
+					  static_cast <pisa_bitdepth_type>
+					  (_bitdepth));
 
-  if ( param->pixeltype == PISA_PT_BW )
+  if (_pixeltype == PISA_PT_BW)
     info->resize_flag	= PISA_RS_NN;
   else
     info->resize_flag	= PISA_RS_BC;
@@ -266,58 +288,57 @@ int scan_manager::init_zoom ( resize_img_info * info, scan_parameter * param )
   return PISA_ERR_SUCCESS;
 }
 
-/*------------------------------------------------------------*/
-int scan_manager::init_moire( moire_img_info * info,
-			      scan_parameter * param )
+int
+scan_manager::init_moire (moire_img_info *info)
 {
-  info->resolution = param->resolution [ 0 ] * param->zoom [ 0 ] / 100;
+  info->resolution = _resolution[0] * _zoom[0] / 100;
 
-  param->resolution [ 0 ] =
-  param->resolution [ 1 ] =
-    iscan::moire::get_res_quote (info->resolution, m_scanner_info.dumb);
+  _resolution[0] =
+  _resolution[1] =
+    iscan::moire::get_res_quote (info->resolution, !has_zoom ());
 
-  if (!m_scanner_info.support_zoom)
+  if (!has_zoom ())
     {
-      long resolution = get_valid_resolution (param->resolution[0]);
-
-      if (resolution)
+      try
 	{
-	  param->resolution[0] = param->resolution[1] = resolution;
+	  get_valid_resolution (&_resolution[0], &_resolution[1]);
+	}
+      catch (const pisa_error& e)
+	{
+	  if (pisa_error (PISA_STATUS_UNSUPPORTED) != e)
+	    throw (e);
+	  // else do nothing
 	}
     }
 
-  info->in_resolution = param->resolution [ 0 ];
+  info->in_resolution = _resolution[0];
 
-  param->zoom [ 0 ] = 100;
-  param->zoom [ 1 ] = 100;
+  _zoom [ 0 ] = 100;
+  _zoom [ 1 ] = 100;
 
-  info->in_width	= ::inch2width ( param->area [ 0 ],
-					 param->resolution [ 0 ] );
-  info->in_height	= ::inch2height ( param->area [ 1 ],
-					 param->resolution [ 1 ] );
-  info->in_rowbytes	= ::calc_rowbytes ( info->in_width,
-					    static_cast <pisa_pixel_type>
-					    ( param->pixeltype ) );
+  info->in_width	= ::inch2width  (_area[0], _resolution[0]);
+  info->in_height	= ::inch2height (_area[1], _resolution[1]);
+  info->in_rowbytes	= calc_rowbytes (info->in_width,
+					 static_cast <pisa_pixel_type>
+					 (_pixeltype));
 
-  info->out_width	= ::inch2width ( param->area [ 0 ],
-					 info->resolution );
-  info->out_height	= ::inch2height ( param->area [ 1 ],
-					 info->resolution );
-  info->out_rowbytes	= ::calc_rowbytes ( info->out_width,
-					    static_cast <pisa_pixel_type>
-					    (param->pixeltype ) );
+  info->out_width	= ::inch2width  (_area[0], info->resolution);
+  info->out_height	= ::inch2height (_area[1], info->resolution);
+  info->out_rowbytes	= calc_rowbytes (info->out_width,
+					 static_cast <pisa_pixel_type>
+					 (_pixeltype));
 
-  info->bits_per_pixel	= ::calc_bitperpix ( static_cast <pisa_pixel_type>
-					     ( param->pixeltype ),
-					     static_cast <pisa_bitdepth_type>
-					     ( param->bitdepth ) );
+  info->bits_per_pixel	= calc_bitperpix (static_cast <pisa_pixel_type>
+					  (_pixeltype),
+					  static_cast <pisa_bitdepth_type>
+					  (_bitdepth));
 
   return PISA_ERR_SUCCESS;
 }
 
 /*------------------------------------------------------------*/
-int scan_manager::init_sharp ( sharp_img_info * info,
-			       scan_parameter * param )
+int
+scan_manager::init_sharp (sharp_img_info *info)
 {
   long resolution;
 
@@ -337,33 +358,28 @@ int scan_manager::init_sharp ( sharp_img_info * info,
     }
   else
     {
-      info->in_width	= ::inch2width ( param->area [ 0 ],
-					 param->resolution [ 0 ],
-					 param->zoom [ 0 ] );
-      info->in_height	= ::inch2height ( param->area [ 1 ],
-					 param->resolution [ 1 ],
-					 param->zoom [ 1 ] );
+      info->in_width	= ::inch2width  (_area[0], _resolution[0], _zoom[0]);
+      info->in_height	= ::inch2height (_area[1], _resolution[1], _zoom[1]);
 
-      resolution	= param->resolution [ 0 ];
+      resolution	= _resolution[0];
     }
 
-  info->in_rowbytes	= ::calc_rowbytes ( info->in_width,
-					    static_cast <pisa_pixel_type>
-					    ( param->pixeltype ) );
+  info->in_rowbytes	= calc_rowbytes (info->in_width,
+					 static_cast <pisa_pixel_type>
+					 (_pixeltype));
   
-  info->bits_per_pixel	= ::calc_bitperpix ( static_cast <pisa_pixel_type>
-					    ( param->pixeltype ),
-					     static_cast <pisa_bitdepth_type> 
-					     ( param->bitdepth ) );
+  info->bits_per_pixel	= calc_bitperpix (static_cast <pisa_pixel_type>
+					  (_pixeltype),
+					  static_cast <pisa_bitdepth_type>
+					  (_bitdepth));
 
   info->out_width	= info->in_width;
   info->out_height	= info->in_height;
   info->out_rowbytes	= info->in_rowbytes;
 
   m_sharp_cls->set_parms (resolution,
-			  param->film == PISA_FT_NEGA
-			  || param->film == PISA_FT_POSI,
-			  m_scanner_info.dumb,
+                          using_tpu (),
+			  !has_zoom (),
 			  & info->strength,
 			  & info->radius,
 			  & info->clipping);
@@ -382,10 +398,16 @@ int scan_manager::modify_img_info ( int * width, int * height )
       if ( m_resize_info.in_width == m_resize_info.out_width )
 	{
 	  m_resize_info.out_width = * width;
+	}
+      else if ( ( has_autocrop () && get_autocrop () )
+                || ( has_size_check () && get_size_check () ) )
+        {
+          m_resize_info.out_width =
+            (*width * m_resize_info.out_width + 0.5) / m_resize_info.in_width;
+        }
 	  m_resize_info.out_rowbytes = ( m_resize_info.out_width *
 					 m_resize_info.bits_per_pixel +
 					 7 ) / 8;
-	}
 
       m_resize_info.in_width	= * width;
       m_resize_info.in_rowbytes	= ( m_resize_info.in_width *
@@ -396,10 +418,50 @@ int scan_manager::modify_img_info ( int * width, int * height )
 	{
 	  m_resize_info.out_height = * height;
 	}
+      else if ( ( has_autocrop () && get_autocrop () )
+                || ( has_size_check () && get_size_check () ) )
+        {
+          m_resize_info.out_height =
+            (*height * m_resize_info.out_height + 0.5) / m_resize_info.in_height;
+        }
 
       m_resize_info.in_height	= * height;
     }
  
+  if ( m_moire )
+    {
+      if ( ( has_autocrop () && get_autocrop () )
+           || ( has_size_check () && get_size_check () ) )
+        {
+          if ( m_moire_info.in_width == m_moire_info.out_width )
+            {
+              m_moire_info.out_width = * width;
+            }
+          else
+            {
+              m_moire_info.out_width =
+                (*width * m_moire_info.resolution + 0.5) / _resolution[0];
+            }
+          if ( m_moire_info.in_height == m_moire_info.out_height )
+            {
+              m_moire_info.out_height = * height;
+            }
+          else
+            {
+              m_moire_info.out_height =
+                (*height * m_moire_info.resolution + 0.5) / _resolution[1];
+            }
+          m_moire_info.in_width  = * width;
+          m_moire_info.in_height = * height;
+          m_moire_info.in_rowbytes =
+            calc_rowbytes (m_moire_info.in_width,
+                           static_cast <pisa_pixel_type> (_pixeltype));
+          m_moire_info.out_rowbytes =
+            calc_rowbytes (m_moire_info.out_width,
+                           static_cast <pisa_pixel_type> (_pixeltype));
+        }
+    }
+
   if ( m_sharp )
     {
       m_sharp_info.in_width	= * width;
@@ -474,7 +536,7 @@ int scan_manager::create_img_cls ( void )
 		     & m_moire_img_info [ _OUT ],
 		     m_moire_info );
 
-      m_moire_cls = new iscan::moire (m_moire_info, m_scanner_info.dumb);
+      m_moire_cls = new iscan::moire (m_moire_info, !has_zoom ());
     }
   
   if ( m_resize )
@@ -507,15 +569,15 @@ int scan_manager::release_memory ( void )
   m_sharp_cls = 0;
 
   if ( m_resize_img_info )
-    delete m_resize_img_info;
+    delete [] m_resize_img_info;
   m_resize_img_info = 0;
 
   if ( m_moire_img_info )
-    delete m_moire_img_info;
+    delete [] m_moire_img_info;
   m_moire_img_info = 0;
 
   if ( m_sharp_img_info )
-    delete m_sharp_img_info;
+    delete [] m_sharp_img_info;
   m_sharp_img_info = 0;
 
   return PISA_ERR_SUCCESS;
@@ -655,10 +717,10 @@ int scan_manager::image_process ( unsigned char * in_img,
 }
 
 void
-scan_manager::adjust_scan_param (long *resolution, long *scale)
+scan_manager::adjust_scan_param (long *resolution, long *scale) const
 {
   int min_res   = 50;
-  int max_res   = m_scanner_info.max_resolution;
+  int max_res   = get_max_resolution ();
   int adj_res   = *resolution;
 
   int min_scale =  50;
@@ -691,44 +753,251 @@ scan_manager::adjust_scan_param (long *resolution, long *scale)
   *scale      = adj_scale;
 }
 
+/*!  Sets main and sub resolutions to the best available value.
 
-SANE_Int
-scan_manager::get_valid_resolution (int min_res)
+     The best available value is the first resolution not smaller than
+     the value passed.  If no such value is available, the largest
+     available resolution will be used unless \a use_max is \c false.
+
+     In the latter case an exception will be thrown instead.
+ */
+void
+scan_manager::get_valid_resolution (long int *x_res, long int *y_res,
+				    bool use_max) const
 {
-  SANE_Int def_res_tbl[] = { 75, 150, 300, 600, 1200 };
-  SANE_Int def_res_cnt   = 4;
+  if (!x_res || !y_res)
+    throw pisa_error (PISA_ERR_PARAMETER);
 
-  const SANE_Int *res_tbl;
-        SANE_Int  res_cnt;
+  SANE_Int res_x = get_resolution (SANE_NAME_SCAN_X_RESOLUTION, *x_res);
+  SANE_Int res_y = get_resolution (SANE_NAME_SCAN_Y_RESOLUTION, *y_res);
 
-  {				// initialize resolution table
-    int opt_id = get_option_id (SANE_NAME_SCAN_RESOLUTION);
-    const SANE_Option_Descriptor *opt_desc
-      = sane_get_option_descriptor (m_hdevice, opt_id);
+  if (0 == res_x || 0 == res_y)
+    {
+      if (!use_max)
+	throw pisa_error (PISA_STATUS_UNSUPPORTED);
+      if (0 == res_x)
+	res_x = get_max_resolution (SANE_NAME_SCAN_X_RESOLUTION);
+      if (0 == res_y)
+	res_y = get_max_resolution (SANE_NAME_SCAN_Y_RESOLUTION);
+    }
 
-    if (opt_desc
-	&& SANE_TYPE_INT             == opt_desc->type
-	&& SANE_CONSTRAINT_WORD_LIST == opt_desc->constraint_type)
-      {
-	res_cnt =   opt_desc->constraint.word_list[0];
-	res_tbl = &(opt_desc->constraint.word_list[1]);
-      }
-    else
-      {
-	res_cnt = def_res_cnt;
-	res_tbl = def_res_tbl;
-
-	if (1200 == m_scanner_info.max_resolution)
-	  ++res_cnt;
-      }
-  }
-
-  {				// find appropriate resolution
-    int i = 0;
-    while (i < res_cnt && res_tbl[i] < min_res)
-      {
-	++i;
-      }
-    return (res_cnt == i ? 0 : res_tbl[i]);
-  }
+  *x_res = res_x;
+  *y_res = res_y;
+}
+
+static bool
+scaling_too_large (const img_size &info)
+{
+  return    info.in_width > SCALING_INPUT_WIDTH_LIMIT
+         && info.in_width != info.out_width;
+}
+
+bool
+scan_manager::area_is_too_large (void) const
+{
+  int area_is_valid;
+
+  // due to a bug in esmod, need to limit the input width when scaling
+  if (m_resize && scaling_too_large (m_resize_info)) return true;
+  if (m_moire  && scaling_too_large (m_moire_info))  return true;
+
+  get_value ("scan-area-is-valid", &area_is_valid);
+  return (0 == area_is_valid);
+}
+
+bool
+scan_manager::adf_duplex_direction_matches (void) const
+{
+  int rv;
+
+  // if option is inactive, assume direction does not match
+  if (!is_activate ("adf-duplex-direction-matches"))
+    return false;
+
+  try
+    {
+      get_value ("adf-duplex-direction-matches", &rv);
+    }
+  catch (pisa_error& oops)
+    { // if no option available, assume direction does not match
+      return false;
+    }
+  return (SANE_TRUE == rv);
+}
+
+void
+scan_manager::has_prev_img ( int has_prev )
+{
+  _has_prev_img = has_prev;
+}
+
+bool
+scan_manager::push_button_needs_polling (void) const
+{
+  SANE_Bool result = false;
+
+  if (is_activate ("needs-polling"))
+    get_value ("needs-polling", &result);
+
+  return result;
+}
+
+void
+scan_manager::update_settings (bool is_preview)
+{
+  SANE_Bool auto_scan = !is_preview;
+
+  set_scan_source ();
+
+  if (is_activate ("adf-auto-scan"))
+    set_value ("adf-auto-scan", &auto_scan);
+
+  _max_descreen_resolution = get_max_resolution (has_zoom () ? 800 : 600);
+
+  set_film_type ();
+  set_focus (_focus);
+  set_speed (_speed);
+  set_deskew (_deskew);
+  set_color_mode (_pixeltype, _bitdepth);
+
+  if (is_preview || _has_prev_img)
+    {
+      set_gamma_table (_gamma.gamma_r);
+      if (_pixeltype == PISA_PT_BW)
+        set_threshold (_threshold);
+    }
+  if (_pixeltype == PISA_PT_RGB)
+    set_color_profile (_coef);
+  set_brightness_method (br_iscan);
+  if (has_brightness())
+    {
+      set_brightness (_brightness);
+    }
+  if (has_contrast())
+    {
+      set_contrast (_contrast);
+    }
+  set_scan_resolution (_resolution[0], _resolution[1]);
+  set_scan_zoom (_zoom[0], _zoom[1]);
+  set_scan_area (_offset[0], _offset[1],
+                 _area[0], _area[1]);
+}
+
+pisa_error_id
+scan_manager::set_scan_parameters (const settings& set, const marquee& marq)
+{
+  _pixeltype	= set.imgtype.pixeltype;
+  _bitdepth	= set.imgtype.bitdepth;
+  _dropout	= set.imgtype.dropout;
+  _monoopt	= set.imgtype.monoopt;
+  _halftone	= set.imgtype.halftone;
+
+  _offset[0]	= marq.offset.x;
+  _offset[1]	= marq.offset.y;
+  _area[0]	= marq.area.x;
+  _area[1]	= marq.area.y;
+  _resolution[0]= (set.resolution * marq.scale + 50) / 100;
+  _resolution[1]= (set.resolution * marq.scale + 50) / 100;
+  _zoom[0]	= 100;
+  _zoom[1]	= 100;
+
+  _brightness	= marq.brightness;
+  _contrast	= marq.contrast;
+  _deskew	= get_deskew();
+
+  for (int i = 0; i < 256; ++i)
+    {
+      _gamma.gamma_r[i] = marq.lut.gamma_r[i];
+      _gamma.gamma_g[i] = marq.lut.gamma_g[i];
+      _gamma.gamma_b[i] = marq.lut.gamma_b[i];
+    }
+
+  if (set.imgtype.pixeltype == PISA_PT_RGB)
+    {
+      generate_color_coef (_coef, set.coef, marq.saturation);
+    }
+  else			// use identity matrix
+    {			// FIXME: fold into generate_color_coef()
+      for (int i = 0; i < 9; ++i)
+	_coef[i] = 0.0;
+
+      _coef[0] = 1.0;
+      _coef[4] = 1.0;
+      _coef[8] = 1.0;
+    }
+
+  _threshold = marq.threshold;
+  _speed = set.enable_draft_mode;
+  _focus = marq.focus;
+
+  if (PISA_PT_BW == set.imgtype.pixeltype)
+    _usm = 0;
+  else
+    _usm = set.usm;
+
+  if (PISA_DESCREEN_ON == set.imgtype.de_screening)
+    _de_screening = 1;
+  else
+    _de_screening = 0;
+
+
+  // check for error conditions
+  pisa_error_id err = PISA_ERR_SUCCESS;
+
+  if (set.imgtype.de_screening == PISA_DESCREEN_ON)
+    {
+      if (   _resolution[0] > _max_descreen_resolution
+	  || _resolution[1] > _max_descreen_resolution)
+	err = PISA_ERR_MRRESTOOHIGH;
+    }
+  return err;
+}
+
+#include "pisa_view_manager.h"
+pisa_error_id
+scan_manager::set_scan_parameters (const settings& set, const marquee& marq,
+				   int resolution)
+{
+  _pixeltype	= PISA_PT_RGB;
+  _bitdepth	= PISA_BD_8;
+  _dropout	= PISA_DO_NONE;
+  _monoopt	= PISA_MO_NONE;
+  _halftone	= PISA_HT_NONE;
+
+  _offset[0]	= marq.offset.x;
+  _offset[1]	= marq.offset.y;
+  _area[0]	= marq.area.x;
+  _area[1]	= marq.area.y;
+  _resolution[0]= resolution;
+  _resolution[1]= resolution;
+  _zoom[0]	= 100;
+  _zoom[1]	= 100;
+
+  _brightness	= marq.brightness;
+  _contrast	= marq.contrast;
+  _deskew	= get_deskew();
+
+  // gamma table
+  for (int i = 0; i < 256; ++i)
+    {
+      _gamma.gamma_r[i] = i;
+      _gamma.gamma_g[i] = i;
+      _gamma.gamma_b[i] = i;
+    }
+
+  // profile matrix
+  for (int i = 0; i < 9; ++i)
+    _coef[i] = 0.0;
+
+  _coef[0] = 1.0;
+  _coef[4] = 1.0;
+  _coef[8] = 1.0;
+
+  _threshold	= 0;
+  _speed	= 1;
+  _focus	= marq.focus;
+  _usm		= 0;
+  _de_screening	= 0;
+
+  return PISA_ERR_SUCCESS;
 }
