@@ -1,6 +1,6 @@
 /* 
    SANE EPSON backend
-   Copyright (C) 2001 SEIKO EPSON Corporation
+   Copyright (C) 2001, 2008  SEIKO EPSON CORPORATION
 
    Date         Author      Reason
    06/01/2001   N.Sasaki    New
@@ -29,7 +29,7 @@
 */
 
 // scan button
-char * scan_xpm[] = {
+const char * scan_xpm[] = {
 "36 28 257 2",
 "  	c None",
 ". 	c #0C0C0C",
@@ -318,7 +318,7 @@ char * scan_xpm[] = {
 "D D D D D D D D D D D D D D D D D D D D D D D D D D D D D D D D D D D D "};
 
 // preview button
-char * preview_xpm[] = {
+const char * preview_xpm[] = {
 "18 18 257 2",
 "  	c None",
 ". 	c #0C0C0C",
@@ -597,7 +597,7 @@ char * preview_xpm[] = {
 "D D d.@.R R R R R R R R R @.D D D D "};
 
 // zoom
-char * zoom_xpm[] = {
+const char * zoom_xpm[] = {
 "18 18 257 2",
 "  	c None",
 ". 	c #0C0C0C",
@@ -876,7 +876,7 @@ char * zoom_xpm[] = {
 "D D D D D D D D D D D D D D D D D D "};
 
 // auto
-char * auto_xpm[] = {
+const char * auto_xpm[] = {
 "18 18 257 2",
 "  	c None",
 ". 	c #0C0C0C",
@@ -1155,7 +1155,7 @@ char * auto_xpm[] = {
 "U.U.U.U.U.U.U.U.U.U.v b U.U.U.U.U.U."};
 
 // gamma correction mono
-char * gamma_m_xpm[] = {
+const char * gamma_m_xpm[] = {
 "30 20 31 1",
 " 	c None",
 ".	c #000000",
@@ -1211,7 +1211,7 @@ char * gamma_m_xpm[] = {
 
 
 // gamma correction red
-char * gamma_r_xpm[] = {
+const char * gamma_r_xpm[] = {
 "30 20 31 1",
 " 	c None",
 ".	c #FF0000",
@@ -1267,7 +1267,7 @@ char * gamma_r_xpm[] = {
 
 
 // gamma correction green
-char * gamma_g_xpm[] = {
+const char * gamma_g_xpm[] = {
 "30 20 31 1",
 " 	c None",
 ".	c #00FF00",
@@ -1323,7 +1323,7 @@ char * gamma_g_xpm[] = {
 
 
 // gamma correction blue
-char * gamma_b_xpm[] = {
+const char * gamma_b_xpm[] = {
 "30 20 31 1",
 " 	c None",
 ".	c #0000FF",
@@ -1378,7 +1378,7 @@ char * gamma_b_xpm[] = {
 ".+@#$%&*=-;>,')!~{]^/(_:<[}|12"};
 
 // penguin
-char * penguin_xpm[] = {
+const char * penguin_xpm[] = {
 "250 101 257 2",
 "  	c None",
 ". 	c #000000",
